package tfg;

import org.jsfml.audio.Sound;
import org.jsfml.audio.SoundBuffer;
import org.jsfml.audio.SoundSource;
import org.jsfml.graphics.Drawable;
import org.jsfml.graphics.RenderStates;
import org.jsfml.graphics.RenderTarget;
import org.jsfml.graphics.Sprite;
import org.jsfml.graphics.Texture;

import java.io.IOException;
import java.nio.file.Paths;

/**
 * Holds everything that pertains to the player (you?).
 * @author Ritchie Cunningham
 */
public class Player extends Entity implements Drawable {
    /**
     * The texture for the sprite.
     */
    private Texture playerSpritesheetTexture = new Texture();
    /**
     * The player sprite that supports animation.
     */
    private final AnimatedSprite playerSprite;
    /**
     * The map the player is currently on.
     */
    private Map currentMap;
    /**
     * The action the player is currently performing.
     */
    private PlayerAction currentAction = PlayerAction.NONE;
    /**
     * The buffer that allows the 'stuck' sound to be loaded.
     */
    private final SoundBuffer cannotMoveBuffer = new SoundBuffer();
    /**
     * The object that plays the 'stuck' sound.
     */
    private final Sound cannotMove = new Sound();
    
    /**
     * Create a new player with a location at (x,y).
     * @param x Starting x pos.
     * @param y Starting y pos.
     */
    public Player(int x, int y) {
        entityLoc = new Location(x, y);
        /* Load sprite texture and 'stuck' sound. */
        try {
            playerSpritesheetTexture.loadFromFile(Paths.get("res/player.png"));
            cannotMoveBuffer.loadFromFile(Paths.get("res/stuck.wav"));
        } catch(IOException ex) {
            ex.printStackTrace();
        }
        Sprite sprite = new Sprite(); /* Create a new regular sprite. */
        sprite.setTexture(playerSpritesheetTexture);
        /* Create a new animated sprite from the regular sprite. */
        playerSprite = new AnimatedSprite(sprite, entityLoc);
        cannotMove.setBuffer(cannotMoveBuffer);
    }

    /**
     * Create a new player at(0,0).
     */
    public Player() {
        this(0,0);
    }

    /**
     * Change the map the is on.
     * @param m New map.
     */
    public void changeMap(Map m) {
        currentMap = m;
    }

    /**
     * Get the map the player is currently on.
     * @return Map player is currently on.
     */
    public Map getMap() {
        return currentMap;
    }

    /**
     * Move the player in the specified direction.
     * @param d Direction to move the player.
     */
    public void move(Direction d) {
        if(currentAction == PlayerAction.NONE) {
            /*
             * Get the location relative to the current location.
             * e.g. NORTH would return the location above the current location.
             */
            Location newLoc = entityLoc.getRelativeLocation(d);
            entityLoc.setDirection(d);
            if(currentMap.isValidLocation(newLoc)) {
                currentAction = PlayerAction.MOVING;
                playerSprite.startAnimation(AnimationType.WALKING);
            } else if(cannotMove.getStatus() == SoundSource.Status.STOPPED) {
                /*
                 * Play the stationary walk animation.
                 * It appears like the player is trying to move, but can't.
                 */
                playerSprite.startAnimation(AnimationType.STATIONARY_WALK);
                /* Play an annoying sound. */
                cannotMove.play();
            }
        }
    }


    /**
     * Update the animation progress.
     */
    public void update() {
        if(currentAction == PlayerAction.MOVING) {
            if(playerSprite.finishedAnimating()) {
                currentAction = PlayerAction.NONE;
                /* Actually move the location. */
                entityLoc = entityLoc.getRelativeLocation(entityLoc.getDirection());
                /* Update the sprite with new location. */
                playerSprite.updatePosition(entityLoc);            
            }
        }
        /* Update the animation, if there is no current animation, ignore. */
        playerSprite.animate();
    }
    
    /**
     * Get animated sprite.
     * @return Animated sprite.
     */
    public AnimatedSprite getAnimatedSprite() {
        return playerSprite;
    }

    /**
     * Draw the player on screen.
     */
    public void draw(RenderTarget target, RenderStates states) {
        /* Get the animated sprite and draw it. */
        playerSprite.getSprite().draw(target, states);
    }
}
